/*
*************************************************************************************************
*		CM60010ACT6 65k Color Graphical LCD Display Driver (Samsung S6B33BC controller)
*										All Rights Reserved
* File name		: s6b33bc.c
* Programmer 	: John Leung, TechToys Co. Hong Kong
* Web presence  : www.TechToys.com.hk
* Note			: 
* Language		: Keil C (C51) compiler, Eval version 7.09 under uVision2 IDE v2.40
* Hardware		: AT89S8253-cLCD-STK1, & CM60010ACT6_BO
* Date			: 2nd Nov 2006 Version 1.0
*************************************************************************************************
*										DESCRIPTION
*
* This software module provides interface to CM60010ACT6 color graphical LCD module of 128*160
* Display controller chip is Samsung S6B33BC
* Module requires 3.3V DC supply to Vdd, LED backlight requires 6.2-6.5V, 15mA typical
*
* The LCD has been mount on a PCB with 2.00mm pinouts, pin id as follows:
* --------- Module															MCU -------------
* pin 1		GND 	- GND													GND
* pin 2		Vdd 	- 3.3V Supply											3.3V
* pin 3		NC (no connect)
* pin 4 	A0 		- (H:Data, L:Command)									P2.1
* pin 5		WR 		- active low, L:Write; use with RD pin					P2.2
* pin 6		RD 		- active low, L:Read; use with WR pin					P2.3
* pin 7-14  D7:D0 	- LCD_DATA 												P0
* pin 15	CS 		- chip select, low active								P2.4
* pin 16	BL_EN 	- backlight enable signal								P2.5
					  (high active, wired to LM2665M6 on the LCD module)
* pin 17	RST 	- reset, low active										P2.6
* pin 18	NC
* pin 19	LED+ 	- LED backlight, requires 6.2-6.5V, 15mA typical		VBL wired to IRLML6401
* pin 20	LED- 	- Cathode for LED-backlight, wired to GND				GND
***********************************************************************************************
*/

#include <REG8253.H>					
#include <delay.h>				
#include "s6b33bc.h"
#if PIXEL_FONT_EN
#include "sysfont.h"	
#endif

/*
*********************************************************************************************************
*											LOCAL DEFINITIONS
*********************************************************************************************************
*/

#define OUTPUT							0
#define INPUT							1
#define HIGH							1
#define LOW								0

#define OSCILLATION_MODE_SET			0x02
#define DRIVER_OUTPUT_MODE_SET			0x10
#define DC_DC_SET						0x20
#define CURRENT_BIAS_SET				0x22
#define PCK_GENER_MODE_SET				0x24	//DC-DC Clock division set
#define DCDC_AMP_ON_OFF_SET				0x26
#define TEMP_COMPENSATION_SET			0x28
#define CONTRAST_CONTROL1				0x2A
#define CONTRAST_CONTROL2				0x2B
#define STANDBY_MODE_OFF				0x2C
#define STANDBY_MODE_ON					0x2D
#define DDRAM_BURST_MODE_OFF			0x2E
#define DDRAM_BURST_MODE_ON				0x2F
#define ADDRESSING_MODE_SET				0x30
#define ROW_VECTOR_MODE_SET				0x32	
#define N_LINE_INVERSION_SET			0x34
#define FRAME_FREQUENCY_CONTROL_SET		0x36
#define ENTRY_MODE_SET					0x40
#define X_ADDRESS_AREA_SET				0x42
#define Y_ADDRESS_AREA_SET				0x43
#define RAM_SKIP_AREA_SET				0x45
#define DISPLAY_OFF						0x50
#define DISPLAY_ON						0x51
#define SPEC_DISPLAY_PATTERN_SET		0x53
#define PARTIAL_DISPLAY_MODE_SET		0x55
#define PARTIAL_DISPLAY_START_LINE_SET	0x56
#define PARTIAL_DISPLAY_END_LINE_SET	0x57
#define AREA_SCROLL_MODE_SET			0x59
#define SCROLL_START_LINE_SET			0x5A
#define SET_DISPLAY_DATA_LENGTH			0xFC

/*
*********************************************************************************************************
*										LOCAL FUNCTIONS PROTOTYPE
*********************************************************************************************************
*/
static void cDispRstArea(void);


/*
*********************************************************************************************************
*                         			RESET DISPLAY AREA TO 128*160 MATRIX
*
* Description : This function resets the display area to 128*160 matrix, local function
* Arguments   : none
*			
* Returns     : none
* Note		  : This function is a local function.
*				Dependency on cDispWrCmd(DRIVER_OUTPUT_MODE_SET) under cDispInit() because
*				display direction changed on SDIR, and CDR.
*********************************************************************************************************
*/
static void cDispRstArea(void)
{
	cDispWrCmd(X_ADDRESS_AREA_SET);			//X-address counter set, offset from 2~161 for 160 dots
	cDispWrCmd(2);
	cDispWrCmd(161);

	cDispWrCmd(Y_ADDRESS_AREA_SET);			//Y-address counter set, offset from 4~131 for 128 dots
	cDispWrCmd(4);
	cDispWrCmd(131);
}

/*
*********************************************************************************************************
*                         				DISPLAY INITIALIZATION
*
* Description : This function initializes the LCD module		
* Arguments   : none
*			
* Returns     : none
* Note		  : This function should be called once before any of the other functions
*********************************************************************************************************
*/
void cDispInit(void)
{
	cDispInitPort();

	cDispReset();

	cDispWrCmd(STANDBY_MODE_OFF);			//standby mode off
	DelayMs(20);							//delay 20ms for 1st booster output stablization

	cDispWrCmd(OSCILLATION_MODE_SET);		//use internal OSC and OSC On
	cDispWrCmd(0x01);
	DelayMs(20);
	
	cDispWrCmd(DC_DC_SET);					//select the first booster's boosting step for V1 gen
	cDispWrCmd(0x0F);						//x1.5 step for normal mode
	DelayMs(20);

	cDispWrCmd(DCDC_AMP_ON_OFF_SET);		//control boosters
	cDispWrCmd(0x01);						//1st booster ON
	DelayMs(20);
	
	cDispWrCmd(DCDC_AMP_ON_OFF_SET);		//1st booster and V1 AMP ON
	cDispWrCmd(0x09);
	DelayMs(20);

	cDispWrCmd(DCDC_AMP_ON_OFF_SET);		//1'st booster, V1 AMP and 2'nd booster ON
	cDispWrCmd(0x0B);						
	DelayMs(20);

	cDispWrCmd(DCDC_AMP_ON_OFF_SET);		//1'st booster, V1 AMP, 2'nd booster and 
	cDispWrCmd(0x0F);						//3'rd booster ON	
	DelayMs(20);

	cDispWrCmd(TEMP_COMPENSATION_SET);		//set the temperature compensation ratio
	cDispWrCmd(0x00);

	cDispWrCmd(DDRAM_BURST_MODE_OFF);		//burst mode off for data RAM write

	cDispWrCmd(RAM_SKIP_AREA_SET);			//RAM address skip area setting
	cDispWrCmd(0x00);

	cDispWrCmd(SPEC_DISPLAY_PATTERN_SET);	//specify display pattern
	cDispWrCmd(0x00);						//normal display
	
	cDispWrCmd(DRIVER_OUTPUT_MODE_SET);		//set the display duty to be 1/162 matching the LCD panel
	cDispWrCmd(0x25);						//SDIR=1, SWP=0, & CDR=1 for display direction matching
											//PIC18-LF4550-STK1 hardware, added in version 1.0a
											//Reference: S6B33BC, page 30
										
	cDispWrCmd(ENTRY_MODE_SET);				//entry mode set, non-reverse, Y-Direction
	cDispWrCmd(0x00);	
					
	cDispWrCmd(CURRENT_BIAS_SET);			//1/5 bias set
	cDispWrCmd(0x11);

	cDispWrCmd(PCK_GENER_MODE_SET);			//DC-DC Clock div set
	cDispWrCmd(0x33);						//fosc/8 and partial mode 1				

	cDispWrCmd(N_LINE_INVERSION_SET);		//N-line inversion set
	cDispWrCmd(0x10);

	cDispWrCmd(FRAME_FREQUENCY_CONTROL_SET);
	cDispWrCmd(0x01);

	cDispWrCmd(SPEC_DISPLAY_PATTERN_SET);	//specify display pattern
	cDispWrCmd(0x00);						//normal display

	cDispWrCmd(PARTIAL_DISPLAY_MODE_SET);	//partial display mode set
	cDispWrCmd(0x00);

	cDispSetContrast1(0x60);				//set a good looking contrast for each module

	cDispRstArea();
	
	cDispWrCmd(ADDRESSING_MODE_SET);		//addressing mode set, 65k color
	cDispWrCmd(0x1C);						

	cDispClr(RGB(31,63,31));				//display white color by mixing R=31,G=63,B=31

	cDispWrCmd(DISPLAY_ON);	
}


/*
*********************************************************************************************************
*                         				DISPLAY ON/OFF SWITCH
*
* Description : This function turns display ON/OFF by writing the command DISPLAY_ON / DISPLAY_OFF
* Arguments   : (BOOLEAN) 'sw' 		ENABLE for turning ON defined under s6b33bc.h
*									DISABLE for turning OFF defined under s6b33bc.h
*				
* Returns     : none
* Notes		  : Internal status and display RAM data not changed by switching between ON and OFF
*********************************************************************************************************
*/
void cDispSwitch(BOOLEAN sw)
{
	(sw==ENABLE)? cDispWrCmd(DISPLAY_ON):cDispWrCmd(DISPLAY_OFF);
}

/*
*********************************************************************************************************
*                         	CLEAR DISPLAY BY WRITING PURE COLOR TO THE WHOLE SCREEN
*
* Description : This function writes a fixed int to all pixels making use of the automatic
*				address increment feature for X and Y address counters
*				
* Arguments   : 'color' is in 16-bit, [R4R3R2R1R0][G5G4G3G2G1G0][B4B3B2B1B0] 
*						Writing color=0xFFFF equivalent to white color
*						color=0x0000 eq. black
*						color=0xF800 eq. red
*						color=0x07E0 eq. green
*						color=0x001F eq. blue
*				
* Returns     : none
*********************************************************************************************************
*/
void cDispClr(INT16U color)
{
	INT8U x,y,hiByte,lowByte;

	hiByte  = color>>8;
	lowByte = (INT8U)color;

		for(x=0;x<MAX_ROW_PIXEL;x++)
		{
			for(y=0;y<MAX_COL_PIXEL;y++)
			{
			cDispWrDat(hiByte);
			cDispWrDat(lowByte);
			}
		}
}


/*
*********************************************************************************************************
*                         				SET PIXEL AT x, y POSITION
*
* Description : This function sets a pixel with (INT16U)color at a position defined by (x,y) following 
*				Cartesian coordinate system. Coordinates (0,0) at the top left corner, 
*				coordinates (127,159) at the lower right corner of the LCD screen, w/ CM60010ACT6_BO
*				in a verticle manner (J1 pins 1-20 verticle, pin 1 upward)
*
* Arguments   : 'x'    		0....MAX_COL_PIXEL-1 is the matrix position in x-axis (left ->right)
*				'y'    		0....MAX_ROW_PIXEL-1 is the matrix position in y-axis (up -> down)
*				'color'  	sets 16-bit color
* Returns     : none
* Notes		  : Should apply the local function cDispRstArea() to reset display area 128*160
*********************************************************************************************************
*/
void cDispSetPixel(INT8U x, INT8U y, INT16U color)
{
	INT8U hiByte, lowByte;

	hiByte  = color>>8;
	lowByte = (INT8U)color;

	cDispWrCmd(X_ADDRESS_AREA_SET);			//X-address area set in the 160 pixel direction (y-axis)
	cDispWrCmd(y+2);						//offset for scan COM from 2~161
	cDispWrCmd(y+3);
	cDispWrCmd(Y_ADDRESS_AREA_SET);			//Y-address area set in the 128 pixel direction (x-axis)
	cDispWrCmd(x+4);						//offset for scan SEG from 4~131
	cDispWrCmd(x+5);

	cDispWrDat(hiByte);
	cDispWrDat(lowByte);	
}


/*
*********************************************************************************************************
*                 CONVERT A STRING FROM DATA MEMORY (RAM) TO PIXEL DATA AND DISPLAY AT x,y
*
* Description : This function outputs a string in RAM in graphic mode (pixel-by-pixel) at a position
*				(x,y) following Cartesian coordinate system with variable font spacing def by fontSpace.
*				Character wrapping is allowed.
*				*** REQUIRE AT LEAST ONE FONT SET (sysfont.h) included under this file ****
*				Cartesian coordinates as (0,0) at the top left hand corner, (127,159) at the 
*				lower right hand corner, w/ CM60010ACT6_BO in a verticle manner (J1 pins 1-20 verticle, 
*				pin 1 upward)
*
* Arguments   : 'x'    		0....MAX_COL_PIXEL-1 is matrix position in horizontal direction (x-axis).
*							
*				'y'    		0....MAX_ROW_PIXEL-1 is matrix position in vertical direction (y-axis).
*
*				'*pStr'		pointer to an ASCII string in RAM, null terminated
*				'fontSpace' font spacing from 1 to 127
*				'color'		16-bit color possible
* Returns     : none
* Notes		  : 
* Example 	  : ....
*				char s[20];
*				main() 
*				{ 
*					cDispPixStrAt(0,20,s,1,RGB(31,0,0)); 	//Display the ASCII string in s[20]
*															//at (x,y)=(0,20), font space = 1
*															//color in red
*				}
*********************************************************************************************************
*/
#if PIXEL_FONT_EN
void cDispStrAt(INT8U x, INT8U y, char *pStr, INT8U fontSpace, INT16U color)
{
	INT8U pixelByte,bit_idx, i;				//loop counters 
	INT8U c,fwidth, fheight;				//character, font width & height store

	while (*pStr)
	{
		c = *pStr++ - 32;
				
		fwidth = SYS_FNT[c].fontWidth;
		fheight = 8;

		if((x+fwidth)>MAX_COL_PIXEL) 		//character wrapping here				
		{
			x=0;
			y+=fheight;
		}

		if((y+fheight)>MAX_ROW_PIXEL)		//check for y boundary				
		{
			y=0;
		}

		for(i=0;i<fheight;i++)
		{
			pixelByte = SYS_FNT[c].fontBody[i];
			for(bit_idx=0; bit_idx<fwidth; bit_idx++)
				{
						if((pixelByte)&(0x80>>bit_idx))
							cDispSetPixel(x+bit_idx, y+i, color);
				}
		}
		
		x = x+fwidth+fontSpace;
	}	

	cDispRstArea();							//reset the display area for 128*160 after DispSetPixel()
}
#endif

/*
*********************************************************************************************************
*                         				WRITE A PATTERN AT X, Y COORDINATES
*
* Description : This function writes a particular pattern of size patWidth and patHeight at a position 
*				defined by (x,y) following Cartesian coordinate system. 
*				Cartesian coordinates as (0,0) at the top left hand corner, (127,159) at the 
*				lower right hand corner, w/ CM60010ACT6_BO in a verticle manner 
*				(J1 pins 1-20 verticle, pin 1 upward)
* Arguments   : 'x'    		0....MAX_COL_PIXEL-1 is matrix position in horizontal direction (x-axis)
*				'y'    		0....MAX_ROW_PIXEL-1 is matrix position in vertical direction (y-axis)
*				'*pPat'		pointer to pattern data
*				'color'  	color in 16-bit mode
* Returns     :	none
* Notes		  : 
*********************************************************************************************************
*/
void cDisprPatAt(INT8U x, INT8U y, INT16U *pPat)
{
 	INT16U   col, row, pWidth, pHeight, pixel;

	pWidth  = pPat[0];
	pHeight = pPat[1];

	for(row=0;row<pHeight;row++)
		{	
			for(col=0;col<pWidth;col++)
			{
				pixel = pPat[col+row*pWidth+2];
				cDispSetPixel(col+x,row+y,pixel);
			}
		}	
	cDispRstArea();							//reset the display area for 128*160 after DispSetPixel()
}

/*
*********************************************************************************************************
*                       ADJUST CONSTRAST IN NORMAL DISPLAY MODE & PARTIAL DISPLAY MODE 0
*
* Description : This function updates the contrast control value in normal display mode and partial 
*				display mode 0
* Arguments   : 'contrast'  0....255
* Returns     :	none
* Notes		  : The relation between V1 voltage and Contrast(1) set value in 3-bit step.
*				Refer to Samsung data sheet (S6B33BC) page 36 for details
*********************************************************************************************************
*/
void cDispSetContrast1(INT8U contrast)
{
	//Contrast control 1 in normal display mode & partial display mode 0
	cDispWrCmd(CONTRAST_CONTROL1);	
	cDispWrCmd(contrast);	
}

/*
*********************************************************************************************************
*                         			LOW LEVEL PORT INITIALIZATION
*
* Description : This function performs low level port initialization
* Arguments   : none
* Returns     : none
* Notes		  : Hardware specific
*********************************************************************************************************
*/
void cDispInitPort(void)
{	
	cDISP_CS		= HIGH;						//de-select the color LCD to start with
	cDISP_A0		= HIGH;
	cDISP_WR		= HIGH;
	cDISP_RD		= HIGH;
	cDISP_DATA		= 0x00;
	//Since the same PCB is used for JHD12864J LCD module as well, there are two gates for backlight control
	//1. cDISP_BL_PWR = power supply controlled by IRLML6401TR
	//2. cDISP_LM2665_SW = charge pump switch built-in the CM60010ACT6_BO module
	cDISP_BL_PWR	= LOW; 	//cDISP_BL_PWR=HIGH -> MOSFET turned ON forever	thus cDISP_BL_PWR appears once
	cDISP_LM2665_SW = HIGH; //cDISP_LM2665_SW=HIGH -> LM2665 turned OFF to trn off backlight
}
/*
*********************************************************************************************************
*                         				LOW LEVEL COMMAND WRITE TO LCD
*
* Description : This function performs low level command write to LCD
* Arguments   : (INT8U) 'cmd' 	is the command written to the LCD module
* Returns     : none
* Notes		  : Hardware specific.
*********************************************************************************************************
*/
void cDispWrCmd(INT8U cmd)
{
	cDISP_A0 	= LOW;			//A0 LOW for command
	cDISP_CS 	= LOW;	
	cDISP_DATA 	= cmd;	
	cDISP_WR 	= LOW;
	cDISP_WR 	= HIGH;															
	cDISP_CS 	= HIGH;
}
/*
*********************************************************************************************************
*                         				LOW LEVEL DATA WRITE TO LCD
*
* Description : This function performs low level display data write to LCD
* Arguments   : (INT8U) 'dat' 	is the data written to the LCD module
* Returns     : none
* Notes		  : Hardware specific.
*********************************************************************************************************
*/
void cDispWrDat(INT8U dat)
{
	cDISP_A0 	= HIGH;		//A0 HIGH for data
	cDISP_CS 	= LOW;	
	cDISP_DATA 	= dat;	
	cDISP_WR 	= LOW;
	cDISP_WR 	= HIGH;															
	cDISP_CS 	= HIGH;
}

/*
*********************************************************************************************************
*                         					cDispBackLite
*
* Description : This function control the LCD backlight ENABLE -OR- DISABLE
* Arguments   : 'BOOLEAN' ctrl 		ENABLE 	for backlight ON
*									DISABLE	for backlight OFF
*			
* Returns     : none
* Notes		  : Hardware specific.
*********************************************************************************************************
*/
void cDispBackLite(BOOLEAN ctrl)
{
	if (ctrl==ENABLE)
	{
		cDISP_LM2665_SW = LOW;	//turn ON LM2665 thus turn ON the backlight
	} 
		else
	{
		cDISP_LM2665_SW = HIGH;	//else, turn off LM2665 and thus turn OFF the backlight
	}
}

/*
*********************************************************************************************************
*                         					cDispReset
*
* Description : This function resets the LCD module
* Arguments   : none
*			
* Returns     : none
* Notes		  : Hardware specific.
*********************************************************************************************************
*/
void cDispReset(void)
{
	cDISP_RST = LOW;
	DelayMs(1);
	cDISP_RST = HIGH;
}
